#!/usr/bin/perl -w
#
# INN nnrpd perl auth module for LDAP
#
#      Copyright (C) 2004  christian mock <cm@tahina.priv.at>
#
#      This program is free software; you can redistribute it and/or modify
#      it under the terms of the GNU General Public License as published by
#      the Free Software Foundation; either version 2 of the License, or
#      (at your option) any later version.
#
#      This program is distributed in the hope that it will be useful,
#      but WITHOUT ANY WARRANTY; without even the implied warranty of
#      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#      GNU General Public License for more details.
#
#      You should have received a copy of the GNU General Public License
#      along with this program; if not, write to the Free Software
#      Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA


require '/usr/local/news/lib/innshellvars.pl';

use strict;
use Net::LDAP;
use Sys::Syslog;

use vars qw(%attributes %authcodes %users $ldap
	    $ldap_host $ldap_bind_template $ldap_ag_entry $current_user
	    %ag_attrs );

$ldap_host = 'ldap.example.com';
$ldap_bind_template = 'cn=%s,ou=newsusers,dc=example,dc=com';

# These codes are a widely implemented de facto standard.
%authcodes = ('allowed' => 281, 'denied' => 502);

# Mappings of LDAP nnrpdAccessGroup attributes to perl hook return keys
%ag_attrs = ( nnrpdNewsgroups => 'newsgroups',
	      nnrpdRead => 'read',
	      nnrpdPost => 'post',
	      nnrpdAccess => 'access',
	      nnrpdKey => 'key',
	      nnrpdRejectWith => 'reject_with',
	      nnrpdMaxRate => 'max_rate',
	      nnrpdLocaltime => 'localtime',
	      nnrpdNewsmaster => 'newsmaster',
	      nnrpdStrippath => 'strippath',
	      nnrpdPerlfilter => 'perlfilter',
	      nnrpdPythonfilter => 'pythonfilter',
	      nnrpdVirtualhost => 'virtualhost',
	      nnrpdPathhost => 'pathhost' );

sub auth_init() {
    openlog('nnrpd-ldap-auth', 'pid', 'news');
    syslog('notice','starting');
}

sub authenticate() {
    my $user = $attributes{'username'};
    my $pass = $attributes{'password'};

    return ($authcodes{denied}, "No username given.")
        unless defined $user;

    my $ldap = ldap_connect();
    unless(defined $ldap) {
	syslog('err', 'LDAP connect error');
	return($authcodes{denied}, 'LDAP connect error (user %s)',
	       $user);
    }
    my $m = $ldap->bind(sprintf($ldap_bind_template, $user), password => $pass);
    if($m->code) {
	syslog('err', 'LDAP bind error for %s: %s', $user, $m->error);
	ldap_disconnect();
	return($authcodes{denied}, '');
    } 
    ## the the user's LDAP entry and store -- the perl access hook
    ## is called immediately after the authenticate hook in
    ## nnrpd/commands.c, so this should be safe...
    $m = $ldap->search(base => sprintf($ldap_bind_template, $user),
		       scope => 'base',
		       filter => '(objectClass=*)');
    if($m->code) {
	syslog('err', 'LDAP search error for %s: %s', $user, $m->error);
	ldap_disconnect();
	return($authcodes{denied}, 'LDAP search error');
    }
    if($m->count != 1) {
	syslog('err', 'LDAP search returned %d entries for %s', $m->count,
	       $user);
	ldap_disconnect();
	return($authcodes{denied}, 'LDAP search result error');
    }
    syslog('notice', 'Authenticated user %s', $user);
    my $ag = $m->entry(0)->get_value("nnrpdAccessGroup");
    unless(defined $ag) {
	syslog('err', 'User %s has no nnrpdAccessGroup attribute', $user);
	ldap_disconnect();
	return($authcodes{denied}, 'LDAP no nnrpdAccessGroup');
    }
    $m = $ldap->search(base => $ag,
		       scope => 'base',
		       filter => '(objectClass=nnrpdAccessGroup)');
    if($m->code) {
	syslog('err', 'LDAP search error for %s: %s', $ag, $m->error);
	ldap_disconnect();
	return($authcodes{denied}, 'LDAP search error (nnrpdAccessGroup)');
    }
    if($m->count != 1) {
	syslog('err', 'LDAP search returned %d entries for %s', $m->count,
	       $ag);
	ldap_disconnect();
	return($authcodes{denied},
	       'LDAP search result error (nnrpdAccessGroup)');
    }
    $ldap_ag_entry = $m->entry(0);
    ldap_disconnect();
    return($authcodes{allowed}, '');
}

## entry point from nnrpd
sub access {
    my %ret;
    foreach my $attr (sort keys %ag_attrs) {
	if($ldap_ag_entry->exists($attr)) {
	    $ret{$ag_attrs{$attr}} = $ldap_ag_entry->get_value($attr);
	    syslog('notice', 'access: %s: %s', $ag_attrs{$attr},
		   $ret{$ag_attrs{$attr}});
	}
    }
    %ret;
}

sub ldap_connect {
    my $l = Net::LDAP->new($ldap_host);
    $l;
}

sub ldap_disconnect {
    undef $ldap;
}

